
/*!
***********************************************************************
*  \file
*      mbuffer.h
*
*  \brief
*      Frame buffer functions
*
*  \author
*      Main contributors (see contributors.h for copyright, address and affiliation details)
*      - Karsten Shring          <suehring@hhi.de>
***********************************************************************
*/
#ifndef _MBUFFER_H_
#define _MBUFFER_H_

#define MAX_LIST_SIZE 33


//! definition a picture (field or frame)
typedef struct storable_picture
{
  PictureStructure structure;

  int         poc;
  int         top_poc;
  int         bottom_poc;
  int         frame_poc;
  int         order_num;
  int64       ref_pic_num[6][MAX_LIST_SIZE];
  int64       frm_ref_pic_num[6][MAX_LIST_SIZE];
  int64       top_ref_pic_num[6][MAX_LIST_SIZE];
  int64       bottom_ref_pic_num[6][MAX_LIST_SIZE];
  unsigned    frame_num;
  int         pic_num;
  int         long_term_pic_num;
  int         long_term_frame_idx;

  int         is_long_term;
  int         used_for_reference;
  int         is_output;
  int         non_existing;

  int         size_x, size_y, size_x_cr, size_y_cr;
  int         chroma_vector_adjustment;
  int         coded_frame;
  int         MbaffFrameFlag;

  imgpel **   imgY;          //!< Y picture component
  imgpel *    imgY_11;       //!< Y picture component with padded borders
  imgpel *    imgY_11_w;     //!< Y picture component with padded borders for weighted prediction
  imgpel **   imgY_ups;      //!< Y picture component upsampled (Quarter pel)
  imgpel **   imgY_ups_w;    //!< Y picture component upsampled (Quarter pel) for weighted prediction
  imgpel ***  imgUV;         //!< U and V picture components
#ifdef ADAPTIVE_FILTER
  imgpel **     imgY_ups_aif;      //!< Y picture component upsampled with new calculated filter coeffitients
  double **     imgY_ups_aif_hor;  //!< Y picture component upsampled with new calculated horizontal filter coefficients (separable aif) 
  imgpel **     p_imgY_ups_aif;    //!< pointer to imgY_ups or to imgY_ups_aif

#ifdef E_DAIF
  imgpel *    imgY_11_aif;       //!< Y picture component with padded borders filtered with new calcualted filter coefficients
  imgpel *    p_imgY_11_aif;    //!< pointer to imgY_11 or to imgY_11_aif
#endif

#endif

#ifdef SWITCHED_FILTERS
  imgpel **   imgY_filt;          //!< Y picture component
  imgpel *    imgY_11_filt;       //!< Y picture component with padded borders
  imgpel *    imgY_11_w_filt;     //!< Y picture component with padded borders for weighted prediction
  imgpel **   p_imgY_filt;  
  imgpel *    p_imgY_11_filt;
  imgpel *    p_imgY_11_w_filt;  
#endif  // SWITCHED_FILTERS

  byte *      mb_field;      //!< field macroblock indicator

  char  ***   ref_idx;       //!< reference picture   [list][subblock_y][subblock_x]

  int64 ***   ref_pic_id;    //!< reference picture identifier [list][subblock_y][subblock_x]
  //   (not  simply index) 

  int64 ***   ref_id;        //!< reference picture identifier [list][subblock_y][subblock_x]
  //   (not  simply index) 

  short ****  mv;            //!< motion vector       [list][subblock_x][subblock_y][component]

  byte **     moving_block;
  byte **     field_frame;         //!< indicates if co_located is field or frame.

  struct storable_picture *top_field;     // for mb aff, if frame for referencing the top field
  struct storable_picture *bottom_field;  // for mb aff, if frame for referencing the bottom field
  struct storable_picture *frame;         // for mb aff, if field for referencing the combined frame

  int         chroma_format_idc;
  int         frame_mbs_only_flag;
  int         frame_cropping_flag;
  int         frame_cropping_rect_left_offset;
  int         frame_cropping_rect_right_offset;
  int         frame_cropping_rect_top_offset;
  int         frame_cropping_rect_bottom_offset;
#ifdef ADAPTIVE_FILTER
  double      FilterCoefficients[15][37];
  int         slice_type;
#endif

#ifdef SINGLE_PASS_AIF

  imgpel **     imgY_ups_aif_spa[MAX_FILTER_NUM];
  double **     imgY_ups_aif_hor_spa[MAX_FILTER_NUM];
#ifdef E_DAIF
  imgpel *    imgY_11_aif_spa[MAX_FILTER_NUM];
#endif

  // The following parameters are the true optimal filter for the current frame.
  // These information can be used by the subsequent frames. --Kai '09
  int HaveAdpCoeff;
  int FilterFlag[ 15 ]; 
  int DiffQFilterCoef[15][37];
  int UseAllSubpelPositions;                  // 1 if FilterFlag for all independent positions is 1
  int SubpelPositionsPattern;

#ifdef E_DAIF
  int FilterFlagInt;
  double FilterCoefInt [26]; 
  int DiffQFilterCoeffInt[26];
  int DiffQFilterOffsetI[15], DiffQFilterOffsetF[15];
  int DiffQFilterOffsetIntI, DiffQFilterOffsetIntF;
#endif

#ifdef EDAIF2
  int RealTimeDecomposition;
  int g_nBitsThresh;

  int Calc2Filt_Indexes[15][37];
  double STANDARD_2D_FILTER[15][37];
  int realSymCommands[4][15];
  int SymmetryPosition[20];
  int POS_EQUATION_NUMBER[20];
  int Filt_Indexes_offset[20];
  int IsDiagonal1D[20];
  int Is1DPosition[20];
  int TwoDSymmetricPattern[15][37];
  int FILTER_NEW_SUB_POS[20];
#endif


  // The following parameters are the filter coefficients carried by the current frame.
  // They may not be the optimal ones. --Kai '09
  int HaveAdpCoeff_Cur;
  int FilterFlag_Cur[ 15 ]; 
  double FilterCoefficients_Cur[15][37];
  int DiffQFilterCoef_Cur[15][37];
  int UseAllSubpelPositions_Cur;                  // 1 if FilterFlag for all independent positions is 1
  int SubpelPositionsPattern_Cur;

#ifdef E_DAIF
  int FilterFlagInt_Cur;
  double FilterCoefInt_Cur [26]; 
  int DiffQFilterCoeffInt_Cur[26];
  int DiffQFilterOffsetI_Cur[15], DiffQFilterOffsetF_Cur[15];
  int DiffQFilterOffsetIntI_Cur, DiffQFilterOffsetIntF_Cur;
#endif

#ifdef EDAIF2
  int RealTimeDecomposition_cur;
  int g_nBitsThresh_cur;

  int Calc2Filt_Indexes_cur[15][37];
  double STANDARD_2D_FILTER_cur[15][37];
  int realSymCommands_cur[4][15];
  int SymmetryPosition_cur[20];
  int POS_EQUATION_NUMBER_cur[20];
  int Filt_Indexes_offset_cur[20];
  int IsDiagonal1D_cur[20];
  int Is1DPosition_cur[20];
  int TwoDSymmetricPattern_cur[15][37];
  int FILTER_NEW_SUB_POS_cur[20];
#endif

#endif

} StorablePicture;


//! definition a picture (field or frame)
typedef struct colocated_params
{
  int         mb_adaptive_frame_field_flag;
  int         size_x, size_y;

  int64       ref_pic_num[6][MAX_LIST_SIZE];  

  char  ***   ref_idx;       //!< reference picture   [list][subblock_y][subblock_x]
  int64 ***   ref_pic_id;    //!< reference picture identifier [list][subblock_y][subblock_x]
  short ****  mv;            //!< motion vector       [list][subblock_x][subblock_y][component]  
  byte **     moving_block;

  // Top field params
  int64       top_ref_pic_num[6][MAX_LIST_SIZE];  
  char  ***   top_ref_idx;       //!< reference picture   [list][subblock_y][subblock_x]
  int64 ***   top_ref_pic_id;    //!< reference picture identifier [list][subblock_y][subblock_x]
  short ****  top_mv;            //!< motion vector       [list][subblock_x][subblock_y][component]  
  byte **     top_moving_block;

  // Bottom field params
  int64       bottom_ref_pic_num[6][MAX_LIST_SIZE];  
  char  ***   bottom_ref_idx;       //!< reference picture   [list][subblock_y][subblock_x]
  int64 ***   bottom_ref_pic_id;    //!< reference picture identifier [list][subblock_y][subblock_x]
  short ****  bottom_mv;            //!< motion vector       [list][subblock_x][subblock_y][component] 
  byte **     bottom_moving_block;

  byte        is_long_term;
  byte **     field_frame;         //!< indicates if co_located is field or frame.

} ColocatedParams;

//! Frame Stores for Decoded Picture Buffer
typedef struct frame_store
{
  int       is_used;                //!< 0=empty; 1=top; 2=bottom; 3=both fields (or frame)
  int       is_reference;           //!< 0=not used for ref; 1=top used; 2=bottom used; 3=both fields (or frame) used
  int       is_long_term;           //!< 0=not used for ref; 1=top used; 2=bottom used; 3=both fields (or frame) used
  int       is_orig_reference;      //!< original marking by nal_ref_idc: 0=not used for ref; 1=top used; 2=bottom used; 3=both fields (or frame) used

  int       is_non_existent;

  unsigned  frame_num;
  int       frame_num_wrap;
  int       long_term_frame_idx;
  int       is_output;
  int       poc;

  StorablePicture *frame;
  StorablePicture *top_field;
  StorablePicture *bottom_field;

} FrameStore;


//! Decoded Picture Buffer
typedef struct decoded_picture_buffer
{
  FrameStore  **fs;
  FrameStore  **fs_ref;
  FrameStore  **fs_ltref;
  unsigned      size;
  unsigned      used_size;
  unsigned      ref_frames_in_buffer;
  unsigned      ltref_frames_in_buffer;
  int           last_output_poc;
  int           max_long_term_pic_idx;

  int           init_done;

  FrameStore   *last_picture;
} DecodedPictureBuffer;


extern DecodedPictureBuffer dpb;
extern StorablePicture **listX[6];
extern int listXsize[6];

void             init_dpb(void);
void             free_dpb(void);
FrameStore*      alloc_frame_store(void);
void             free_frame_store(FrameStore* f);
StorablePicture* alloc_storable_picture(PictureStructure type, int size_x, int size_y, int size_x_cr, int size_y_cr);
void             free_storable_picture(StorablePicture* p);
void             store_picture_in_dpb(StorablePicture* p);
void             replace_top_pic_with_frame(StorablePicture* p);
void             flush_dpb(void);

void             dpb_split_field(FrameStore *fs);
void             dpb_combine_field(FrameStore *fs);
void             dpb_combine_field_yuv(FrameStore *fs);

void             init_lists(int currSliceType, PictureStructure currPicStructure);
void             reorder_ref_pic_list(StorablePicture **list, int *list_size, 
                                      int num_ref_idx_lX_active_minus1, int *reordering_of_pic_nums_idc, 
                                      int *abs_diff_pic_num_minus1, int *long_term_pic_idx);

void             init_mbaff_lists(void);
void             alloc_ref_pic_list_reordering_buffer(Slice *currSlice);
void             free_ref_pic_list_reordering_buffer(Slice *currSlice);

void             fill_frame_num_gap(ImageParameters *img);

ColocatedParams* alloc_colocated(int size_x, int size_y,int mb_adaptive_frame_field_flag);
void free_colocated(ColocatedParams* p);
void compute_colocated(ColocatedParams* p, StorablePicture **listX[6]);

#endif

